#!/usr/bin/env python

# Python bindings to the Google search engine
# Copyright (c) 2009-2015, Mario Vilas
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice,this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

__all__ = ['search']

import os
import sys
import time

if sys.version_info[0] > 2:
    from http.cookiejar import LWPCookieJar
    from urllib.request import Request, urlopen
    from urllib.parse import quote_plus, urlparse, parse_qs
else:
    from cookielib import LWPCookieJar
    from urllib import quote_plus
    from urllib2 import Request, urlopen
    from urlparse import urlparse, parse_qs

# Lazy import of BeautifulSoup.
BeautifulSoup = None

# URL templates to make Google searches.
url_home = "http://www.google.%(tld)s/"
url_search = "http://www.google.%(tld)s/search?hl=%(lang)s&q=%(query)s&btnG=Google+Search&tbs=%(tbs)s&safe=%(safe)s&tbm=%(tpe)s"
url_next_page = "http://www.google.%(tld)s/search?hl=%(lang)s&q=%(query)s&start=%(start)d&tbs=%(tbs)s&safe=%(safe)s&tbm=%(tpe)s"
url_search_num = "http://www.google.%(tld)s/search?hl=%(lang)s&q=%(query)s&num=%(num)d&btnG=Google+Search&tbs=%(tbs)s&safe=%(safe)s&tbm=%(tpe)s"
url_next_page_num = "http://www.google.%(tld)s/search?hl=%(lang)s&q=%(query)s&num=%(num)d&start=%(start)d&tbs=%(tbs)s&safe=%(safe)s&tbm=%(tpe)s"

# Cookie jar. Stored at the user's home folder.
home_folder = os.getenv('HOME')
if not home_folder:
    home_folder = os.getenv('USERHOME')
    if not home_folder:
        home_folder = '.'   # Use the current folder on error.
cookie_jar = LWPCookieJar(os.path.join(home_folder, '.google-cookie'))
try:
    cookie_jar.load()
except Exception:
    pass


# Request the given URL and return the response page, using the cookie jar.
def get_page(url):
    """
    Request the given URL and return the response page, using the cookie jar.

    @type  url: str
    @param url: URL to retrieve.

    @rtype:  str
    @return: Web page retrieved for the given URL.

    @raise IOError: An exception is raised on error.
    @raise urllib2.URLError: An exception is raised on error.
    @raise urllib2.HTTPError: An exception is raised on error.
    """
    request = Request(url)
    request.add_header('User-Agent',
                       'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0)')
    cookie_jar.add_cookie_header(request)
    response = urlopen(request)
    cookie_jar.extract_cookies(response, request)
    html = response.read()
    response.close()
    cookie_jar.save()
    return html


# Filter links found in the Google result pages HTML code.
# Returns None if the link doesn't yield a valid result.
def filter_result(link):
    try:

        # Valid results are absolute URLs not pointing to a Google domain
        # like images.google.com or googleusercontent.com
        o = urlparse(link, 'http')
        if o.netloc and 'google' not in o.netloc:
            return link

        # Decode hidden URLs.
        if link.startswith('/url?'):
            link = parse_qs(o.query)['q'][0]

            # Valid results are absolute URLs not pointing to a Google domain
            # like images.google.com or googleusercontent.com
            o = urlparse(link, 'http')
            if o.netloc and 'google' not in o.netloc:
                return link

    # Otherwise, or on error, return None.
    except Exception:
        pass
    return None


# Shortcut to search images
# Beware, this does not return the image link.
def search_images(query, tld='com', lang='en', tbs='0', safe='off', num=10, start=0,
                  stop=None, pause=2.0, only_standard=False, extra_params={}):
    return search(query, tld, lang, tbs, safe, num, start, stop, pause, only_standard, extra_params, tpe='isch')


# Shortcut to search news
def search_news(query, tld='com', lang='en', tbs='0', safe='off', num=10, start=0,
                stop=None, pause=2.0, only_standard=False, extra_params={}):
    return search(query, tld, lang, tbs, safe, num, start, stop, pause, only_standard, extra_params, tpe='nws')


# Shortcut to search videos
def search_videos(query, tld='com', lang='en', tbs='0', safe='off', num=10, start=0,
                  stop=None, pause=2.0, only_standard=False, extra_params={}):
    return search(query, tld, lang, tbs, safe, num, start, stop, pause, only_standard, extra_params, tpe='vid')


# Shortcut to search shop
def search_shop(query, tld='com', lang='en', tbs='0', safe='off', num=10, start=0,
                stop=None, pause=2.0, only_standard=False, extra_params={}):
    return search(query, tld, lang, tbs, safe, num, start, stop, pause, only_standard, extra_params, tpe='shop')


# Shortcut to search books
def search_books(query, tld='com', lang='en', tbs='0', safe='off', num=10, start=0,
                 stop=None, pause=2.0, only_standard=False, extra_params={}):
    return search(query, tld, lang, tbs, safe, num, start, stop, pause, only_standard, extra_params, tpe='bks')


# Shortcut to search apps
def search_apps(query, tld='com', lang='en', tbs='0', safe='off', num=10, start=0,
                stop=None, pause=2.0, only_standard=False, extra_params={}):
    return search(query, tld, lang, tbs, safe, num, start, stop, pause, only_standard, extra_params, tpe='app')


# Returns a generator that yields URLs.
def search(query, tld='com', lang='en', tbs='0', safe='off', num=10, start=0,
           stop=None, pause=2.0, only_standard=False, extra_params={}, tpe=''):
    """
    Search the given query string using Google.

    @type  query: str
    @param query: Query string. Must NOT be url-encoded.

    @type  tld: str
    @param tld: Top level domain.

    @type  lang: str
    @param lang: Languaje.

    @type  tbs: str
    @param tbs: Time limits (i.e "qdr:h" => last hour, "qdr:d" => last 24 hours, "qdr:m" => last month).

    @type  safe: str
    @param safe: Safe search.

    @type  num: int
    @param num: Number of results per page.

    @type  start: int
    @param start: First result to retrieve.

    @type  stop: int
    @param stop: Last result to retrieve.
        Use C{None} to keep searching forever.

    @type  pause: float
    @param pause: Lapse to wait between HTTP requests.
        A lapse too long will make the search slow, but a lapse too short may
        cause Google to block your IP. Your mileage may vary!

    @type  only_standard: bool
    @param only_standard: If C{True}, only returns the standard results from
        each page. If C{False}, it returns every possible link from each page,
        except for those that point back to Google itself. Defaults to C{False}
        for backwards compatibility with older versions of this module.

    @type  extra_params: dict
    @param extra_params: A dictionary of extra HTTP GET parameters, which must be URL encoded.
        For example if you don't want google to filter similar results you can set the extra_params to
        {'filter': '0'} which will append '&filter=0' to every query.

    @type  tpe: str
    @param tpe: Search type (images, videos, news, shopping, books, apps)
            Use the following values {videos: 'vid', images: 'isch', news: 'nws',
                                      shopping: 'shop', books: 'bks', applications: 'app'}

    @rtype:  generator
    @return: Generator (iterator) that yields found URLs. If the C{stop}
        parameter is C{None} the iterator will loop forever.
    """

    # Lazy import of BeautifulSoup.
    # Try to use BeautifulSoup 4 if available, fall back to 3 otherwise.
    global BeautifulSoup
    if BeautifulSoup is None:
        try:
            from bs4 import BeautifulSoup
        except ImportError:
            from BeautifulSoup import BeautifulSoup

    # Set of hashes for the results found.
    # This is used to avoid repeated results.
    hashes = set()

    # Prepare the search string.
    query = quote_plus(query)

    # Check extra_params for overlapping
    for builtin_param in ('hl', 'q', 'btnG', 'tbs', 'safe', 'tbm'):
        if builtin_param in extra_params.keys():
            raise ValueError(
                'GET parameter "%s" is overlapping with \
                the built-in GET parameter',
                builtin_param
            )

    # Grab the cookie from the home page.
    get_page(url_home % vars())

    # Prepare the URL of the first request.
    if start:
        if num == 10:
            url = url_next_page % vars()
        else:
            url = url_next_page_num % vars()
    else:
        if num == 10:
            url = url_search % vars()
        else:
            url = url_search_num % vars()

    # Loop until we reach the maximum result, if any (otherwise, loop forever).
    while not stop or start < stop:

        try:  # Is it python<3?
            iter_extra_params = extra_params.iteritems()
        except AttributeError:  # Or python>3?
            iter_extra_params = extra_params.items()
        # Append extra GET_parameters to URL
        for k, v in iter_extra_params:
            url += url + ('&%s=%s' % (k, v))

        # Sleep between requests.
        time.sleep(pause)

        # Request the Google Search results page.
        html = get_page(url)

        # Parse the response and process every anchored URL.
        soup = BeautifulSoup(html, 'html.parser')
        anchors = soup.find(id='search').findAll('a')
        for a in anchors:

            # Leave only the "standard" results if requested.
            # Otherwise grab all possible links.
            if only_standard and (
                    not a.parent or a.parent.name.lower() != "h3"):
                continue

            # Get the URL from the anchor tag.
            try:
                link = a['href']
            except KeyError:
                continue

            # Filter invalid links and links pointing to Google itself.
            link = filter_result(link)
            if not link:
                continue

            # Discard repeated results.
            h = hash(link)
            if h in hashes:
                continue
            hashes.add(h)

            # Yield the result.
            yield link

        # End if there are no more results.
        if not soup.find(id='nav'):
            break

        # Prepare the URL for the next request.
        start += num
        if num == 10:
            url = url_next_page % vars()
        else:
            url = url_next_page_num % vars()





class Settings:

    def __init__(self):
        pass

    # default settings

    master_title = 'Mr.'
    master_name = 'Semih'
    master_surname = 'Yagcioglu'
    master_gender = 'female'
    master_formal_address = 'sir'
    master_email_username = None
    master_email_password = None
    jarvy_name = 'jarvy'
    jarvy_gender = 'female'

    # messages
    sorry_messages = ['I am sorry, I could not understand that', 'Oh oh',
                      'I am afraid, I can\'t talk about that',
                      'I am thinking but this beats me...', 'Allright allright, I give up', 'Who knows',
                      'Don\'t get me wrong but I don\'t know all the answers',
                      'Hey, why don\'t you ask me another question while I think more about that?',
                      'Believe me I am trying but, I simply could not answer that',
                      'I think I do not understand you',
                      'Hmmm, I am doing my best,just be patient, OK?']
    farewell_messages = ['good bye', 'bye', 'see you', 'farewell']
    positive_answers = ['Yes', 'Of course', 'Definitely']
    negative_answers = ['No', 'Nope', 'I don\'t think so', 'Sorry']
    personal_message_for_jarvy = ['you', 'yourself']
    personal_message_for_master = ['me', 'myself', 'I']
    rudimentary_question_tags = ['who', 'why', 'when', 'where', 'what', 'which', 'how']

    # algorithm parameters
    number_of_search_results = 10
    trusted_source_treshold = 3
    number_of_minimum_sentences = 3  # should reconsider this





import re
import random
from random import randint
import time
import logging
import os
import urllib2
from bs4 import BeautifulSoup

from packages.google import search

from actions import *
from settings import *

"""
Jarvy : Python Intelligent Assistant for Humans
=================

Jarvy, aims to help humans by trying to understand them and figuring out best ways to respond to them.

"""

__title__ = 'jarvy'
__version__ = '1.3.0'
__build__ = 0x010300  # in the format of 00-00-00
__author__ = 'Semih Yagcioglu'
__license__ = 'MIT'
__copyright__ = 'Copyright 2015 Semih Yagcioglu'


class Jarvy:
    """ Jarvy : Python Intelligent Assistant for Humans
    """

    __LOG_LOCATION__ = "log/jarvy.log"

    def __init__(self):

        self.__setupLogger__()
        self.actions = Actions()
        self.settings = Settings()
        self.status = True

        self.wakeup()

    def __setupLogger__(self):

        self.logger = logging.getLogger('Jarvy')
        self.logger.setLevel(logging.INFO)

        # Log formatting
        formatter = logging.Formatter('%(asctime)s\t%(name)s\t%(levelname)s\t%(message)s')

        # Console handler
        # ch = logging.StreamHandler()
        # ch.setFormatter(formatter)
        # self.logger.addHandler(ch)

        # Logfile handler
        directory = os.path.dirname(self.__LOG_LOCATION__)
        if not os.path.exists(directory):
            os.makedirs(directory)

        fh = logging.FileHandler(self.__LOG_LOCATION__)
        fh.setFormatter(formatter)
        self.logger.addHandler(fh)

    # starts jarvy
    def wakeup(self):

        welcome = self.hi()
        print '\n\n' + welcome + "."
        self.logger.info("Woke up.")

        while self.status:
            try:
                message = str(raw_input("What can I help you with?\n\n")).lower()

                if any(m in message for m in self.settings.farewell_messages) and any(m in message for m in self.settings.jarvy_name):
                    self.sleep()
                else:
                    answer = self.answer(message)
                    print answer
            except KeyboardInterrupt:
                self.sleep()

    # terminates jarvy
    def sleep(self):

        farewell = self.farewell()
        print farewell
        self.status = False

        self.logger.info("Slept.")

    # produce salutation message
    def hi(self):

        chance = random.uniform(0, 100)

        if chance <= 20:
            greeting = 'Hello ' + self.settings.master_name
        elif chance <= 30:
            greeting = 'Hi ' + self.settings.master_name
        elif chance <= 40:
            greeting = 'Hey'
        else:
            hour = int(time.strftime('%H', time.localtime()))
            if 0 <= hour < 5:
                greeting = 'Good night ' + self.settings.master_formal_address
            elif 5 <= hour < 12:
                greeting = 'Good morning ' + self.settings.master_formal_address
            elif 12 <= hour < 17:
                greeting = 'Good afternoon ' + self.settings.master_formal_address
            else:
                greeting = 'Good evening ' + self.settings.master_formal_address

        return greeting

    # produce farewell message
    def farewell(self):

        chance = random.uniform(0, 100)

        if chance <= 20:
            farewell = 'Good bye ' + self.settings.master_name
        elif chance <= 40:
            farewell = 'Farewell my friend'
        else:
            hour = int(time.strftime('%H', time.localtime()))
            if 0 <= hour < 5:
                farewell = 'Good night ' + self.settings.master_formal_address
            elif 5 <= hour < 12:
                farewell = 'Have a good day ' + self.settings.master_formal_address
            elif 12 <= hour < 17:
                farewell = 'Good afternoon ' + self.settings.master_formal_address
            else:
                farewell = 'Good evening ' + self.settings.master_formal_address

        return farewell

    # respond to the query message
    def answer(self, message):

        action = self.understand(message)
        responses = self.think(action, message)
        answer = self.explain(responses)

        return answer

    def understand(self, message):

        if any(m in message.split() for m in self.settings.personal_message_for_jarvy):    # it is something about the jarvy
            action = self.actions.about_jarvy
        elif message == self.settings.jarvy_name.lower():  # direct address to jarvy
            action = self.actions.direct_address
        elif any(m.lower() in message.split() for m in self.settings.personal_message_for_master):   # it is something about the master
            action = self.actions.about_master
        elif any(m.lower() in message.split() for m in self.settings.rudimentary_question_tags):     # rewrite message as a search query
            action = self.actions.search_google
        else:
            action = self.actions.say_sorry
        return action

    def think(self, action, message):

        if action == self.actions.about_jarvy:
            responses = self.respond('jarvy')
        elif action == self.actions.direct_address:
            responses = self.respond('direct_address')
        elif action == self.actions.about_master:
            responses = self.respond('master')
        elif action == self.actions.search_google:  # get results from google
            responses = self.make_search(message, 'google')
        elif action == self.actions.search_wolfram:  # get results from wolfram
            responses = self.make_search(message, 'wolfram')
        elif action == self.actions.search_wikipedia:  # get results from wikipedia
            responses = self.make_search(message, 'wikipedia')
        else:
            responses = []

        if not responses:
            rand = randint(0, len(self.settings.sorry_messages) - 1)
            responses = [self.settings.sorry_messages[rand]]

        return responses

    def make_search(self, query, source):

        query = self.rewrite_question(source, query)

        search_results = []
        trusted_results = []

        try:
            urls = search(query, stop=self.settings.number_of_search_results)  # make a search

            for i, url in enumerate(urls):
                search_results.append(url)
                if url.find('wikipedia') > 0 and i < self.settings.trusted_source_treshold:  # trusted source, skip if not in there
                    trusted_results.append(url)
            try:
                if len(trusted_results) > 0:
                    html = urllib2.urlopen(trusted_results[0]).read()
                    soup = BeautifulSoup(html, 'html.parser')
                    p = soup.find('div', id="bodyContent").p
                    soup_p = BeautifulSoup(str(p), 'html.parser')
                    text = soup_p.get_text()
                else:
                    html = urllib2.urlopen(search_results[0]).read()
                    soup = BeautifulSoup(html, 'html.parser')
                    p = soup.find_all('p')
                    soup_p = BeautifulSoup(str(p), 'html.parser')
                    text = soup_p.get_text()
                    pat = re.compile(r'([A-Z][^\.!?]*[\.!?])', re.M)  # pattern to detect sentences
                    matches = pat.findall(text)

                    if len(matches) > self.settings.number_of_minimum_sentences:
                        text = ''.join(matches[0:self.settings.number_of_minimum_sentences])
                    else:
                        text = ''.join(matches[0:])  # TODO: what if the answer is = ''

                responses = [text]
            except:
                responses = []  # there is no proper response, but need to reconsider this
        except:
            responses = []  # no internet access

        return responses

    def explain(self, responses):

        if len(responses) >= 1:  # if there is only one response
            answer = responses[0]
        else:
            pass  # TODO: scoring might be done here!

        return answer

    def respond(self, about):

        if about == 'jarvy':
            responses = ['I am your friend.']
        elif about == 'direct_address':
            responses = ['Yes ' + self.settings.master_formal_address]
        elif about == 'master':
            responses = ['I\'m afraid, I can\'t answer that.']
        else:
            responses = ['']

        return responses

    def rewrite_question(self, source, message):

        query = message  # by default, set it to message

        if source == 'google':
            for r in self.settings.rudimentary_question_tags:
                query = query.replace(r, '')

            query = query.replace(' is ', ' ')
            query = query.replace(' are ', ' ')
            query = query.strip()

        return query


def start():

    j = Jarvy()
    return j

if __name__ == '__main__':
    start()




import unittest


class SimpleTest(unittest.TestCase):
    """Jarvy test cases."""

    def test_starting_out(self):
        self.assertEqual(1, 1)

    def setUp(self):
        import jarvy

    def tearDown(self):
        pass


def main():
    unittest.main()

if __name__ == '__main__':
    main()





class Actions:

    def __init__(self):
        pass

    about_jarvy = 1
    direct_address = 2
    about_master = 3
    search_google = 4
    search_wolfram = 5
    search_wikipedia = 6
    say_sorry = 7


# def enum(**enums):
#     return type('Enum', (), enums)
#
# actions = enum(about_jarvy=1,
#                direct_address=2,
#                about_master=3,
#                search_google=4,
#                search_wolfram=5,
#                search_wikipedia=6,
#                say_sorry=7
#                )



#!/usr/bin/env python

import os
import re
import sys

from codecs import open

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

packages = [
    'jarvy',
    'jarvy.packages',
]

requires = ['beautifulsoup4']

version = ''
with open('jarvy/__init__.py', 'r') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fd.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Cannot find version information')

with open('README.rst', 'r', 'utf-8') as f:
    readme = f.read()
with open('HISTORY.rst', 'r', 'utf-8') as f:
    history = f.read()

setup(
    name='jarvy',
    version=version,
    description='Python Intelligent Assistant for Humans.',
    long_description=readme + '\n\n' + history,
    author='Semih Yagcioglu',
    author_email='semihyagcioglu@yahoo.com',
    url='http://github.com/jarvy/jarvy',
    packages=packages,
    package_data={'': ['LICENSE']},
    package_dir={'jarvy': 'jarvy'},
    include_package_data=True,
    install_requires=requires,
    license='MIT',
    zip_safe=False,
    classifiers=(
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7'
    ),
)
